@@ -3,7 +3,7 @@
 """
 Conversion of iCalendar files to and from RDF representations.
 
-Copyright (C) 2005, 2006 Paul Boddie <paul@boddie.org.uk>
+Copyright (C) 2005, 2006, 2007 Paul Boddie <paul@boddie.org.uk>
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -153,11 +153,16 @@ class Reader:
 
     "A simple class wrapping a file, providing simple pushback capabilities."
 
-    def __init__(self, f):
+    def __init__(self, f, non_standard_newline=0):
 
-        "Initialise the object with the file 'f'."
+        """
+        Initialise the object with the file 'f'. If 'non_standard_newline' is
+        set to a true value (unlike the default), lines ending with CR will be
+        treated as complete lines.
+        """
 
         self.f = f
+        self.non_standard_newline = non_standard_newline
         self.lines = []
         self.line_number = 0
 
@@ -184,7 +189,9 @@ class Reader:
         else:
             # NOTE: Sanity check for broken lines (\r instead of \r\n or \n).
             line = self.f.readline()
-            if line.endswith("\r"):
+            while line.endswith("\r") and not self.non_standard_newline:
+                line += self.f.readline()
+            if line.endswith("\r") and self.non_standard_newline:
                 return line + "\n"
             else:
                 return line
@@ -583,7 +590,7 @@ class DOMParser:
 
 # Public functions.
 
-def parse(f, store, name=None, uriref=None):
+def parse(f, store, name=None, uriref=None, non_standard_newline=0):
 
     """
     Parse the resource data found through the use of the file object 'f', which
@@ -595,11 +602,15 @@ def parse(f, store, name=None, uriref=None):
     the name of the imported resource and to override the unique identifier of
     the resource with a specific URI reference.
 
+    The optional 'non_standard_newline' can be set to a true value (unlike the
+    default) in order to attempt to process files with CR as the end of line
+    character.
+
     As a result of parsing the resource, the root node of the imported resource
     is returned.
     """
 
-    reader = Reader(f)
+    reader = Reader(f, non_standard_newline=non_standard_newline)
     parser = iCalendarParser()
     return parser.parse(reader, store, name=name, uriref=uriref)
 
